"""
Utilities for generating Flash cross-domain policy files.

"""

import xml.dom


minidom = xml.dom.getDOMImplementation('minidom')

def new_policy():
    """
    Create and return a new policy file, as a ``minidom.Document``
    object.
    
    """
    policy_type = minidom.createDocumentType(qualifiedName='cross-domain-policy',
                                             publicId=None,
                                             systemId='http://www.adobe.com/xml/dtds/cross-domain-policy.dtd')
    return minidom.createDocument(None, 'cross-domain-policy', policy_type)

def allow_access_from(policy, domain, to_ports=None, secure=None):
    """
    Insert a new ``allow-access-from`` element into ``policy``,
    allowing access from ``domain``.

    ``domain`` may be any valid domain value, but due to security
    concerns it is strongly recommended that wildcard domains be
    avoided.

    """
    domain_element = policy.createElement('allow-access-from')
    domain_element.setAttribute('domain', domain)
    policy.documentElement.appendChild(domain_element)

def site_control(policy, permitted):
    """
    Insert a ``site-control`` element into ``policy``, using
    ``permitted`` as the value of the
    ``permitted-cross-domain-policies`` attribute.
    
    """
    control_element = policy.createElement('site-control')
    control_element.setAttribute('permitted-cross-domain-policies', permitted)
    policy.documentElement.appendChild(control_element)

def simple_policy(domains):
    """
    Given a list of valid domain values, create a simple policy file
    allowing access from those domains and return the resulting XML as
    a ``minidom.Document`` object.
    
    """
    policy = new_policy()
    for domain in domains:
        allow_access_from(policy, domain)
    return policy

def no_access_policy():
    """
    Create a policy file which permits no access of any sort and
    return the resulting XML as a ``minidom.Document`` object.
    
    """
    policy = new_policy()
    site_control(policy, permitted='none')
    return policy



import xml.dom.minidom

from django.test import TestCase

from flashpolicies import policies


class PolicyGeneratorTests(TestCase):
    """
    Tests for the policy-file generation utilities.
    
    """
    def test_policy_type(self):
        """
        Test that the correct ``DOCTYPE`` declaration is generated.
        
        """
        policy = policies.new_policy()
        self.assertEqual(policy.doctype.systemId, 'http://www.adobe.com/xml/dtds/cross-domain-policy.dtd')
        self.assertEqual(policy.doctype.name, 'cross-domain-policy')
        self.assertEqual(len(policy.childNodes), 2)

    def test_policy_root_element(self):
        """
        Test that the correct root element is inserted.
        
        """
        policy = policies.new_policy()
        self.assertEqual(policy.documentElement.tagName, 'cross-domain-policy')
        self.assertEqual(len(policy.documentElement.childNodes), 0)

    def test_allow_access_domain(self):
        """
        Test that adding access for a domain inserts the proper
        element and attributes.
        
        """
        policy = policies.new_policy()
        policies.allow_access_from(policy, 'media.example.com')
        self.assertEqual(len(policy.documentElement.childNodes), 1)
        self.assertEqual(len(policy.getElementsByTagName('allow-access-from')), 1)
        access_elem = policy.getElementsByTagName('allow-access-from')[0]
        self.assertEqual(len(access_elem.attributes), 1)
        self.assertEqual(access_elem.getAttribute('domain'), 'media.example.com')

    def test_site_control(self):
        """
        Test that adding meta-policy information inserts the proper
        element and attributes.
        
        """
        for permitted in ('none', 'master-only', 'by-content-type',
                          'by-ftp-filename', 'all'):
            policy = policies.new_policy()
            policies.site_control(policy, permitted)
            self.assertEqual(len(policy.documentElement.childNodes), 1)
            self.assertEqual(len(policy.getElementsByTagName('site-control')), 1)
            control_elem = policy.getElementsByTagName('site-control')[0]
            self.assertEqual(len(control_elem.attributes), 1)
            self.assertEqual(control_elem.getAttribute('permitted-cross-domain-policies'), permitted)

    def test_simple_policy(self):
        """
        Test that creating a simple policy with a list of domains
        returns a correct policy document.
        
        """
        domains = ['media.example.com', 'api.example.com']
        policy = policies.simple_policy(domains)
        self.assertEqual(len(policy.documentElement.childNodes), 2)
        self.assertEqual(len(policy.getElementsByTagName('allow-access-from')), 2)
        domain_elems = policy.getElementsByTagName('allow-access-from')
        for i, domain in enumerate(domains):
            self.assertEqual(domain,
                             policy.documentElement.getElementsByTagName('allow-access-from')[i].getAttribute('domain'))

    def test_no_access_policy(self):
        """
        Test that creating a policy which permits no access returns a
        correct policy document.
        
        """
        policy = policies.no_access_policy()
        self.assertEqual(len(policy.documentElement.childNodes), 1)
        self.assertEqual(len(policy.getElementsByTagName('site-control')), 1)
        control_elem = policy.getElementsByTagName('site-control')[0]
        self.assertEqual(control_elem.getAttribute('permitted-cross-domain-policies'), 'none')


class PolicyViewTests(TestCase):
    urls = 'flashpolicies.test_urls'

    def test_simple(self):
        response = self.client.get('/crossdomain1.xml')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'text/x-cross-domain-policy')
        policy = xml.dom.minidom.parseString(response.content)
        domains = ['media.example.com', 'api.example.com']
        self.assertEqual(len(policy.getElementsByTagName('allow-access-from')), 2)
        domain_elems = policy.getElementsByTagName('allow-access-from')
        for i, domain in enumerate(domains):
            self.assertEqual(domain,
                             policy.documentElement.getElementsByTagName('allow-access-from')[i].getAttribute('domain'))

    def test_no_access(self):
        response = self.client.get('/crossdomain2.xml')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'text/x-cross-domain-policy')
        policy = xml.dom.minidom.parseString(response.content)
        self.assertEqual(len(policy.getElementsByTagName('site-control')), 1)
        self.assertEqual(policy.getElementsByTagName('site-control')[0].getAttribute('permitted-cross-domain-policies'),
                         'none')



from django.http import HttpResponse

from flashpolicies import policies


def simple(request, domains):
    """
    A simple Flash cross-domain access policy.

    **Required arguments:**

    ``domains``
        A list of domains from which to allow access. Each value may
        be either a domain name (e.g., ``example.com``) or a wildcard
        (e.g., ``*.example.com``). Due to serious potential security
        issues, it is strongly recommended that you not use wildcard
        domain values.

    **Optional arguments:**

    None.
    
    """
    return HttpResponse(policies.simple_policy(domains).toprettyxml(encoding='utf-8'),
                        content_type='text/x-cross-domain-policy')

def no_access(request):
    """
    A Flash cross-domain access policy which permits no access of any
    kind.
    
    **Required arguments:**

    None.

    **Optional arguments:**

    None.
    
    """
    return HttpResponse(policies.no_access_policy().toprettyxml(encoding='utf-8'),
                        content_type='text/x-cross-domain-policy')



"""
These URLs are used by the test suite to exercise the various
views. You should not use these URLs in any sort of real deployment
situation.

"""

from django.conf.urls.defaults import *


urlpatterns = patterns('',
                       url(r'^crossdomain1.xml$',
                           'flashpolicies.views.simple',
                           { 'domains': ['media.example.com', 'api.example.com'] }),
                       url(r'^crossdomain2.xml$',
                           'flashpolicies.views.no_access'),
                       )
